const truncateMiddle = require('truncate-middle')
const rand = require('randomatic')
const { ts } = require('./../common/logging')

const globalDefaults = {
  timeout: 0
}

const DEBUG_HEADER = true       // change to true for debugging BaseAPIClient headers
const DEBUG_BODY = true         // change to true for debugging BaseAPIClient body
const DEBUG_BODY_MAX_LEN = 6000 // change to limit the max size of a response body

class BaseAPIClient {
  constructor(baseUrl, defaults = {}) {
    this.defaults = Object.assign({ headers: {} }, defaults, { baseUrl }, globalDefaults)
  }

  execute(method, route = '', options = {}) {
    console.log('executing action: ', method, route)
    const start = (new Date()) * 1
    const rid = rand('AAAAAAAA')

    if (DEBUG_HEADER) {
      console.log(ts(), '      >', rid, method, route)
      if (DEBUG_BODY) {
        if (options.qs) {
          console.log(ts(), '      >', rid, 'QUERY', JSON.stringify(options.qs))
        }
        if (options.form) {
          const half = DEBUG_BODY_MAX_LEN / 2
          const truncated = truncateMiddle(JSON.stringify(options.form), half - 2, half - 3, ' ... ')
          console.log(ts(), '      >', rid, 'FORM', truncated)
        }
        if (options.body) {
          const half = DEBUG_BODY_MAX_LEN / 2
          const truncated = truncateMiddle(JSON.stringify(options.body), half - 2, half - 3, ' ... ')
          console.log(ts(), '      >', rid, 'BODY', truncated)
        }
      }
    }

    options.headers = options.headers || {}
    options.headers['X-Thinkigo-Integration-Request-Id'] = rid

    const overrides = { method, uri: route }
    const reqHeaders = Object.assign(this.defaults.headers, options.headers)
    const params = Object.assign(options, this.defaults, { headers: reqHeaders }, overrides)
    
    console.log('action params: ', params)
    const promise = fetch(route, params)
    console.log('action promise: ', promise)
    if (DEBUG_HEADER) {
      const debug = (dcolor, dmsg, body) => {
        const end = (new Date()) * 1
        const duration = end - start
        const color = duration >= 1000 ? 'red' : (duration >= 500 ? 'yellow' : 'green')
        console.log(ts(), '      <', rid, method, route)
        if (DEBUG_BODY) {
          const half = DEBUG_BODY_MAX_LEN / 2
          const truncated = truncateMiddle(JSON.stringify(body || null), half - 2, half - 3, ' ... ')
          console.log(ts(), '      <', rid, truncated)
        }
      }
      promise.then(resp => {
        debug('green', '✔', resp)
      }, resp => {
        const body = resp.error || resp
        const headers = resp.response && resp.response.headers
        debug('red', '✖', body, headers)
      })
    }

    return promise
  }

  query(method, route, qs = {}) {
    Object.keys(qs).forEach(key => {
      if (Array.isArray(qs[key])) {
        qs[key] = JSON.stringify(qs[key])
      }
    })
    return this.execute(method, route, { qs })
  }

  form(method, route, form = {}) {
    return this.execute(method, route, { form })
  }

  get(route, qs = {}) {
    return this.query('GET', route, qs)
  }

  head(route, qs = {}) {
    return this.query('HEAD', route, qs)
  }

  post(route, form = {}) {
    return this.form('POST', route, form)
  }

  put(route, form = {}) {
    return this.form('PUT', route, form)
  }

  patch(route, form = {}) {
    return this.form('PATCH', route, form)
  }

  delete(route, form = {}) {
    return this.form('DELETE', route, form)
  }
}

module.exports = BaseAPIClient
