const imaps = require('imap-simple')
const settings = require('./settings').gmail

const GMAIL_HOST = 'imap.gmail.com'
const GMAIL_PORT = 993
const GMAIL_TLS = true
const GMAIL_INBOX = 'INBOX'
 
class GmailClient {
  constructor(username = settings.username, domain = settings.domain, password = settings.password) {
    this.emailAddress = `${username}@${domain}`
    this.password = password
    this.connection = null
  }

  config(authTimeout = 3000) {
    return {
      imap: {
        user: this.emailAddress,
        password: this.password,
        host: GMAIL_HOST,
        port: GMAIL_PORT,
        tls: GMAIL_TLS,
        authTimeout
      }
    }
  }

  connect(authTimeout = 3000, reconnect = false) {
    return new Promise((resolve, reject) => {
      if (this.connection && !reconnect) {
        resolve(this)
      } else {
        imaps.connect(this.config(authTimeout)).then(connection => {
          this.connection = connection
          resolve(this)
        }, reject)
      }
    })
  }

  fetchEmailsMatchingCriteria(criteria, inbox = GMAIL_INBOX) {
    return this.connection.openBox(inbox).then(() => {
      const options = { bodies: ['TEXT', 'HEADER.FIELDS (FROM TO SUBJECT DATE)'] }
      return this.connection.search(criteria, options)
    }).then(messages => messages)
  }

  fetchRecentEmailsWithSubject(subject, recipient = '', box = GMAIL_INBOX) {
    const yesterday = new Date()
    yesterday.setDate(yesterday.getDate() - 1)
    const criteria = ['ALL', ['HEADER', 'SUBJECT', subject], ['SINCE', yesterday]]
    if (/.+@.+/.test(recipient)) {
      criteria.push(['HEADER', 'TO', recipient])
    }
    return this.fetchEmailsMatchingCriteria(criteria, box)
  }

  static fetchRecentEmailsWithSubject(subject, {
      email = `${settings.username}@${settings.domain}`,
      password = settings.password
    } = {}) {
    const [username, domain] = email.split('@', 2)
    return new GmailClient(username, domain, password).connect()
      .then(gmail => gmail.fetchRecentEmailsWithSubject(subject))
  }

  static fetchRecentEmailsWithSubjectAndRecipient(subject, recipient, {
    password = settings.password
  } = {}) {
    const [username, domain] = recipient.split('@', 2)
    return new GmailClient(username, domain, password).connect()
      .then(gmail => gmail.fetchRecentEmailsWithSubject(subject, recipient))
  }
}

module.exports = GmailClient
