const twilio = require('twilio')
const settings = require('./settings').twilio

class TwilioClient {
  constructor(accountSID = settings.account_sid, authToken = settings.auth_token) {
    this.transport = twilio(accountSID, authToken)
  }

  getIncomingPhoneNumbers() {
    return new Promise((resolve, reject) => {
      this.transport.incomingPhoneNumbers.list((err, response) => {
        if (err) reject(err)
        else resolve(response)
      })
    })
  }

  getMessageList({ to, from, dateSent }) {
    return new Promise((resolve, reject) => {
      this.transport.messages.list({ to, from, dateSent }, (err, response) => {
        if (err) reject(err)
        else resolve(response)
      })
    })
  }

  getTodaysMessageListTo(phone) {
    const today = (new Date()).toISOString().split('T')[0]
    return this.getMessageList({ dateSent: today, to: phone })
  }

  static getTodaysMessageListTo({
    phone = settings.twilio.number,
    accountSID = settings.account_sid,
    authToken = settings.auth_token
  } = {}) {
    return new TwilioClient(accountSID, authToken).getTodaysMessageListTo(phone)
  }
}

module.exports = TwilioClient
